@@ -1,8 +1,8 @@
 """
 MIT License
 
-Copyright (c) 2020 Dominik Kopczynski   -   dominik.kopczynski {at} isas.de
-                   Nils Hoffmann  -  nils.hoffmann {at} isas.de
+Copyright (c) 2020 Dominik Kopczynski   -   dominik.kopczynski {at} univie.ac.at
+                   Nils Hoffmann  -  nils.hoffmann {at} cebitec.uni-bielefeld.de
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -28,7 +28,7 @@ from enum import Enum
 from os import path
 
 import pygoslin
-from pygoslin.domain.LipidExceptions import LipidParsingException, LipidException
+from pygoslin.domain.LipidExceptions import LipidParsingException, LipidException, ParserException
 from itertools import combinations as iter_combinations
 from pygoslin.parser.SumFormulaParserEventHandler import SumFormulaParserEventHandler
 
@@ -103,7 +103,7 @@ class Parser:
             n = self.next_free_rule_index
             self.next_free_rule_index += 1
             return n
-        raise Exception("Error: grammar is too big.")
+        raise ParserException("Error: grammar is too big.")
     
     
     
@@ -123,6 +123,7 @@ class Parser:
         self.OTtoNT = {}
         self.right_pair = []
         self.left_pair = []
+        self.error_message = ""
         
         if grammar_filename == None: return
         
@@ -134,19 +135,18 @@ class Parser:
             ruleToNT = {Parser.EOF_RULE_NAME: Parser.EOF_RULE}
             self.TtoNT[Parser.EOF_SIGN] = set([Parser.EOF_RULE])
             for rule_line in rules:
-                
                 tokens_level_1 = []
                 for t in Parser.split_string(rule_line, Parser.RULE_ASSIGNMENT, self.quote): tokens_level_1.append(t.strip(' '))
-                if len(tokens_level_1) != 2: raise Exception("Error: corrupted token in grammar rule: '%s'" % rule_line);
+                if len(tokens_level_1) != 2: raise ParserException("Error: corrupted token in grammar rule: '%s'" % rule_line);
                 
                 if len(Parser.split_string(tokens_level_1[0], ' ', self.quote)) > 1:
-                    raise Exception("Error: several rule names on left hand side in grammar rule: '%s'" % rule_line);
+                    raise ParserException("Error: several rule names on left hand side in grammar rule: '%s'" % rule_line);
                 
 
                 rule = tokens_level_1[0]
                 
                 if rule ==  Parser.EOF_RULE_NAME:
-                    raise Exception("Error: rule name is not allowed to be called EOF")
+                    raise ParserException("Error: rule name is not allowed to be called EOF")
                 
                 products = [p.strip(' ') for p in Parser.split_string(tokens_level_1[1], Parser.RULE_SEPARATOR, self.quote)]
                 
@@ -157,6 +157,8 @@ class Parser:
                 
                 
                 for product in products:
+                    if len(product) == 0:
+                        raise ParserException("Error: empty product rule in rule: \"%s\"" % rule_line)
                     non_terminals, non_terminal_rules = [], []
                     for NT in Parser.split_string(product, ' ', self.quote):
                         stripedNT = NT.strip(' ')
@@ -214,7 +216,7 @@ class Parser:
                     elif len(non_terminal_rules) == 1:
                         rule_index_1 = non_terminal_rules[0]
                         if rule_index_1 == new_rule_index:
-                            raise Exception("Error: corrupted token in grammar: rule '%s' is not allowed to refer soleley to itself." % rule)
+                            raise ParserException("Error: corrupted token in grammar: rule '%s' is not allowed to refer soleley to itself." % rule)
                         
                         if rule_index_1 not in self.NTtoNT: self.NTtoNT[rule_index_1] = set()
                         self.NTtoNT[rule_index_1].add(new_rule_index)
@@ -227,7 +229,7 @@ class Parser:
             self.parser_event_handler.sanity_check()
             
         else:
-            raise Exception("Error: file '%s' does not exist or can not be opened." % grammar_filename)
+            raise ParserException("Error: file '%s' does not exist or can not be opened." % grammar_filename)
         
         def top_nodes( rule_index):
             collection, collection_top = [rule_index], []
@@ -243,20 +245,27 @@ class Parser:
         
         
         
-        
+        visited = set()
         for rule_index, values in self.NTtoNT.items():
-            for rule in values:
-                for rule_top in top_nodes(rule):
-                    chain = self.collect_backwards(rule, rule_top)
-                    if chain == None: continue
-                    chain = [rule_top] + chain + [rule]
-                    while len(chain) > 1:
-                        top = chain[0]
-                        chain = chain[1:]
-                        self.substitution[rule_index + (top << 16)] = chain
+            rules_to_visit = (set(values) | set([rule_index])) - visited
+            for rule in rules_to_visit:
+                visited.add(rule)
+                
+                for rule_top in set(top_nodes(rule)):
+                    chains = self.collect_backwards(rule, rule_top)
+                    if chains == None: continue
+                    
+                    for chain in chains:
+                        while len(chain) > 1:
+                            top = chain[0]
+                            chain = chain[1:]
+                            self.substitution[(rule_index, top)] = chain
+                    
                     
         for k, v in self.TtoNT.items():
             self.OTtoNT[k] = list(v)[0]
+            
+            
         
         
         for d in [self.TtoNT, self.NTtoNT]:
@@ -276,12 +285,15 @@ class Parser:
             self.left_pair[key & self.MASK].add(key)
            
            
-           
+
+    
+    
+    
     
     def extract_text_based_rules(grammar_filename, quote = DEFAULT_QUOTE):
         grammar, sb,current_position = "", [], 0
         
-        with open(grammar_filename, mode = "rt") as infile:
+        with open(grammar_filename, mode = "rt", encoding= "utf-8") as infile:
             grammar = infile.read() + "\n";
         grammar_length = len(grammar)
         
@@ -340,24 +352,24 @@ class Parser:
             sb.append(grammar[current_position : grammar_length])
             
         else:
-            raise Exception("Error: corrupted grammar '%s', ends either in comment or quote" % grammar_filename)
+            raise ParserException("Error: corrupted grammar '%s', ends either in comment or quote" % grammar_filename)
         
         grammar = "".join(sb).replace("\r\n", "").replace("\n", "").replace("\r", "").strip(" ");
         if grammar[-1] != Parser.RULE_TERMINAL:
-            raise Exception("Error: corrupted grammar'%s', last rule has no termininating sign, was: '%s'" % (grammar_filename, grammar[-1]))
+            raise ParserException("Error: corrupted grammar'%s', last rule has no termininating sign, was: '%s'" % (grammar_filename, grammar[-1]))
         
         rules = Parser.split_string(grammar, Parser.RULE_TERMINAL, quote)
         
         if len(rules) < 1:
-            raise Exception("Error: corrupted grammar '%s', grammar is empty" % grammar_filename)
+            raise ParserException("Error: corrupted grammar '%s', grammar is empty" % grammar_filename)
         
         grammar_name_rule = Parser.split_string(rules[0], ' ', quote)
         
         if grammar_name_rule[0] != "grammar":
-            raise Exception("Error: first rule must start with the keyword 'grammar'")
+            raise ParserException("Error: first rule must start with the keyword 'grammar'")
         
         elif len(grammar_name_rule) != 2:
-            raise Exception("Error: incorrect first rule")
+            raise ParserException("Error: incorrect first rule")
         
         return rules
     
@@ -365,7 +377,7 @@ class Parser:
     
     
     def dump_child(self, class_name, parser_event_handler_name):
-        with open("%s.py" % class_name, mode = "wt") as dumpfile:
+        with open("%s.py" % class_name, mode = "wt", encoding= "utf-8") as dumpfile:
             dumpfile.write("from pygoslin.parser.Parser import Parser\n")
             dumpfile.write("from pygoslin.parser.%s import %s\n\n" % (parser_event_handler_name, parser_event_handler_name))
             dumpfile.write("class %s(Parser):\n" % class_name)
@@ -384,14 +396,14 @@ class Parser:
     
     
     
-    def split_string(text, separator, quote = DEFAULT_QUOTE):
+    def split_string(text, separator, quote = DEFAULT_QUOTE, with_empty = False):
         in_quote, tokens, sb, last_char, last_escaped_backslash = False, [], [], '\0', False
         
         for c in text:
             escaped_backslash = False
             if not in_quote:
                 if c == separator:
-                    if len(sb) > 0: tokens.append("".join(sb))
+                    if len(sb) > 0 or with_empty: tokens.append("".join(sb))
                     sb = []
                 else:
                     if c == quote: in_quote = not in_quote
@@ -407,9 +419,9 @@ class Parser:
             last_escaped_backslash = escaped_backslash
             last_char = c
                 
-        if len(sb) > 0:
+        if len(sb) > 0 or (last_char == ',' and with_empty):
             tokens.append("".join(sb))
-        if in_quote: raise Exception("Error: corrupted token in grammar")
+        if in_quote: raise ParserException("Error: corrupted token in grammar")
         
         return tokens
     
@@ -483,19 +495,28 @@ class Parser:
     
     
     
-    def collect_backwards(self, child_rule_index, parent_rule_index):
-        if child_rule_index not in self.NTtoNT: return None
+    def collect_backwards(self, child_rule_index, parent_rule_index, visited = None, path = None, collection = None):
+        # provides all single linkage paths from a child rule to a parent rule,
+        # and yes, there can be several paths
         
-        for previous_index in self.NTtoNT[child_rule_index]:
-            if previous_index == parent_rule_index:
-                return []
-            
-            elif previous_index in self.NTtoNT:
-                collection = self.collect_backwards(previous_index, parent_rule_index)
-                if collection != None:
-                    collection.append(previous_index)
-                    return collection
-        return None
+        if visited == None: visited, path, collection = set(), [], []
+        
+        if child_rule_index not in self.NTtoNT: return collection
+        visited.add(child_rule_index)
+        path.append(child_rule_index)
+        
+        for previous_rule in self.NTtoNT[child_rule_index]:
+            if previous_rule not in visited:
+                
+                if previous_rule == parent_rule_index:
+                    collection.append(list(path + [parent_rule_index])[::-1])
+                
+                else:
+                    collection = self.collect_backwards(previous_rule, parent_rule_index, visited, path, collection)
+                      
+        path.pop()
+        visited.remove(child_rule_index)
+        return collection
         
         
         
@@ -528,8 +549,9 @@ class Parser:
             top_rule = parse_content[2]
             bottom_rule = self.OTtoNT[parse_content[0]]
         
-        
-        s_key = bottom_rule + (top_rule << 16)
+
+            
+        s_key = (bottom_rule, top_rule)
         if bottom_rule != top_rule and s_key in self.substitution:
             for rule_index in self.substitution[s_key]:
                 node.left = TreeNode(rule_index, rule_index in self.NTtoRule)
@@ -549,12 +571,17 @@ class Parser:
     
     
     def parse(self, text_to_parse, raise_error = True):
+        self.error_message = ""
+        text_to_parse = text_to_parse.strip(" ")
         old_lipid = text_to_parse
         if self.used_eof: text_to_parse += Parser.EOF_SIGN
         
-        self.parse_regular(text_to_parse)
-        if raise_error and not self.word_in_grammar:
-            raise LipidParsingException("Lipid '%s' can not be parsed by grammar '%s'" % (old_lipid, self.grammar_name))
+        try:
+            self.parse_regular(text_to_parse)
+            if raise_error and not self.word_in_grammar:
+                raise LipidParsingException("Lipid '%s' can not be parsed by grammar '%s'" % (old_lipid, self.grammar_name))
+        except Exception as ex:
+            if raise_error: raise ex
         
         return self.parser_event_handler.content
         
@@ -581,7 +608,19 @@ class Parser:
                 break
             if self.used_eof: break
             
-            
+        # search for longest parsed prefix
+        if not self.word_in_grammar:
+            for i in range(len(text_to_parse) - 1, 0, -1):
+                if len(dp_table[i][0]) > 0:
+                    first_rule = list(dp_table[i][0].keys())[0]
+                    parse_tree = TreeNode(first_rule, first_rule in self.NTtoRule)
+                    self.fill_tree(parse_tree, dp_table[i][0][first_rule])
+                    self.error_message = parse_tree.get_text()
+                    break
+                
+                
+    def get_error_message(self):
+        return self.error_message
             
             
             
@@ -592,6 +631,7 @@ class Parser:
         lft = self.left_pair
         
         
+        
         DP = [[{} for j in range(n - i)] for i in range(n)]
         DL = [[set() for j in range(n - i)] for i in range(n)]
         DR = [[set() for j in range(n - i)] for i in range(n)]
@@ -636,6 +676,7 @@ class SumFormulaParser(Parser):
     def __init__(self):
         self.event_handler = SumFormulaParserEventHandler()
         dir_name = path.dirname(pygoslin.__file__)
-        super().__init__(self.event_handler, dir_name + "/data/goslin/SumFormula.g4", Parser.DEFAULT_QUOTE)
+        file_name = path.join(dir_name, "data", "goslin", "SumFormula.g4")
+        super().__init__(self.event_handler, file_name, Parser.DEFAULT_QUOTE)
 
         
